@@ -1,5 +1,5 @@
 /* example3.c	Example code showing how to use Libidn.
- * Copyright (C) 2002  Simon Josefsson
+ * Copyright (C) 2002, 2003  Simon Josefsson
  *
  * This file is part of GNU Libidn.
  *
@@ -32,8 +32,8 @@
  * $ ./example3
  * Input domain encoded as `ISO-8859-1': www.rksmrgs.example
  * Read string (length 23): 77 77 77 2e 72 e4 6b 73 6d f6 72 67 e5 73 aa 2e 65 78 61 6d 70 6c 65 
- * ACE label (length 33): 'www.iesg--rksmrgsa-0zap8p.example'
- * 77 77 77 2e 69 65 73 67 2d 2d 72 6b 73 6d 72 67 73 61 2d 30 7a 61 70 38 70 2e 65 78 61 6d 70 6c 65 
+ * ACE label (length 33): 'www.xn--rksmrgsa-0zap8p.example'
+ * 77 77 77 2e 78 6e 2d 2d 72 6b 73 6d 72 67 73 61 2d 30 7a 61 70 38 70 2e 65 78 61 6d 70 6c 65 
  * $
  *
  */
@@ -43,7 +43,8 @@ main (int argc, char *argv[])
 {
   char buf[BUFSIZ];
   char *p;
-  int rc, i;
+  int rc;
+  size_t i;
 
   printf ("Input domain encoded as `%s': ", stringprep_locale_charset ());
   fflush (stdout);
